Package: fit.models
Version: 0.64
Date: 2020-08-02
Title: Compare Fitted Models
Description: The fit.models function and its associated methods (coefficients, print,
  summary, plot, etc.) were originally provided in the robust package to compare robustly
  and classically fitted model objects. See chapters 2, 3, and 5 in Insightful (2002)
  'Robust Library User's Guide' <http://robust.r-forge.r-project.org/Robust.pdf>). The aim
  of the fit.models package is to separate this fitted model object comparison functionality
  from the robust package and to extend it to support fitting methods (e.g., classical,
  robust, Bayesian, regularized, etc.) more generally.
Authors@R: person("Kjell", "Konis", role = c("aut", "cre"), email = "kjellk@gmail.com")
Author: Kjell Konis [aut, cre]
Maintainer: Kjell Konis <kjellk@gmail.com>
Imports: lattice, stats
Suggests: MASS
License: GPL
RoxygenNote: 7.1.0
NeedsCompilation: no
Packaged: 2020-08-02 13:59:06 UTC; kjellk
Repository: CRAN
Date/Publication: 2020-08-02 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 16:07:51 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(center,default)
S3method(coef,glmfm)
S3method(coef,lmfm)
S3method(designMD,default)
S3method(plot,covfm)
S3method(plot,glmfm)
S3method(plot,lmfm)
S3method(print,covfm)
S3method(print,glmfm)
S3method(print,lmfm)
S3method(print,summary.covfm)
S3method(print,summary.glmfm)
S3method(print,summary.lmfm)
S3method(summary,covfm)
S3method(summary,glmfm)
S3method(summary,lmfm)
S3method(vcor,default)
S3method(vcov,default)
export(center)
export(ddPlot.covfm)
export(designMD)
export(distancePlot.covfm)
export(ellipsesPlot.covfm)
export(fit.models)
export(fmclass.add.class)
export(fmclass.register)
export(overlaidKernelDensityPlot)
export(overlaidQQPlot)
export(overlaidSimpleRegressionPlot)
export(screePlot.covfm)
export(sideBySideIndexPlot)
export(sideBySideKernelDensityPlot)
export(sideBySideQQPlot)
export(sideBySideScatterPlot)
export(vcor)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(lattice,densityplot)
importFrom(lattice,llines)
importFrom(lattice,panel.abline)
importFrom(lattice,panel.densityplot)
importFrom(lattice,panel.loess)
importFrom(lattice,panel.rug)
importFrom(lattice,panel.text)
importFrom(lattice,panel.xyplot)
importFrom(lattice,qqmath)
importFrom(lattice,simpleKey)
importFrom(lattice,strip.default)
importFrom(lattice,xyplot)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cov2cor)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,mahalanobis)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,menu)

export(plot.glmfm)

#' Distance - Distance Plot
#'
#' @description For a \code{covfm} object containing 2 models, this function plots the
#'              Mahalanobis distance from the first model on the y-axis and the
#'              Mahalanobis distance from the second model on the x-axis.
#'
#' @param x a \code{"covfm"} object.
#' 
#' @param level a single numeric value between 0 and 1 giving the
#'              chi-squared percent point used to compute the outlyingness threshold.
#'
#' @param strip a character string printed in the \dQuote{strip} at the top
#'              of the plot.
#'
#' @param id.n a single nonnegative integer specifying the number of
#'             extreme points to label in the plot.
#' 
#' @param ... additional arguments are passed to \code{xyplot}.
#'
#' @return if the models can be compared then the plotted \code{trellis} object is
#'         invisibly returned.  Otherwise \code{x} is returned invisibly.
#'
#' @export
ddPlot.covfm <- function(x, level = 0.95, strip = "", id.n = 3, ...)
{
  n.models <- length(x)
  mod.names <- names(x)
         
  if(n.models == 2) {
    p <- sapply(x, function(u) dim(u$cov)[1])
    n <- length(x[[1]]$dist)

    tdf <- data.frame(x = sqrt(x[[2]]$dist), 
                      y = sqrt(x[[1]]$dist))

    prepanel.special <- function(x, y, thresh) {
      lim <- c(0.0, max(c(x, y, 1.25 * thresh)))
      list(xlim = lim, ylim = lim)
    }

    panel.special <- function(x, y, thresh, id.n, ...) {
      panel.xyplot(x, y, ...)

      if(id.n > 0) {
        out <- which(x > thresh[1] | y > thresh[2])
        id.n <- ifelse(length(out) > id.n, id.n, length(out))
        out <- out[order(x[out]^2 + y[out]^2, decreasing = TRUE)][1:id.n]

        if(length(out))
          panel.text(x[out], y[out], paste(" ", out, sep = ""), adj = 0)
      }

      panel.abline(h = thresh[2], lty = 2)
      panel.abline(v = thresh[1], lty = 2)
      panel.abline(c(0, 1), lty = 4)
      invisible()
    }

    #' @importFrom stats qchisq
    p <- xyplot(y ~ x | strip,
                data = tdf,
                aspect = "iso",
                panel = panel.special,
                prepanel = prepanel.special,
                thresh = sqrt(qchisq(level, p)),
                id.n = id.n,
                strip = function(...) strip.default(...,style = 1),
                ...)

    print(p)
    return(invisible(p))
  }

  invisible(x)
}

#' Register Comparable Functions
#' 
#' The \code{fit.models} package maintains a list of comparable models.
#' These functions provide an api to modify this list.
#' 
#' See the package vignette.
#' 
#' @aliases fmclass.register fmclass.add.class
#' @param fmclass a character string naming the fit.models class to be added.
#' @param classes a character vector naming one or more classes that can be
#' compared by the methods defined for the fit.models class in \code{fmclass}.
#' @param class a character string specifying a class compatible with the
#' methods of \code{fmclass}.
#' @param validation.function a function returning \code{TRUE} when the models
#' are comparable.
#' @param warn a logical value. If TRUE, a warning is printed if \code{class}
#' is already registered.
#' @return a null value is invisibly returned.
#' @keywords misc

#' @rdname fmclass
#' @export
fmclass.register <- function(fmclass, classes, validation.function = NULL)
{
  if(fmclass %in% names(e$fmreg)) {
    message(fmclass, " is already registered in the fit.models registry")
    return(invisible())
  }

  e$fmreg[[fmclass]] <- list(fmclass = fmclass,
                             classes = classes,
                             validation.function = validation.function)

  invisible()
}


#' @rdname fmclass
#' @export
fmclass.add.class <- function(fmclass, class, warn = TRUE)
{
  if(class %in% e$fmreg[[fmclass]]$classes && warn) {
    message(class, " is already registered in the fit.models registry")
    return(invisible())
  }
  
  e$fmreg[[fmclass]]$classes <- union(e$fmreg[[fmclass]]$classes, class)
  
  invisible()
}

#' Fit dot Models
#' 
#' Fit a statistical model using different estimators (e.g., robust and
#' least-squares) or combine fitted models into a single object.  Generic
#' methods then produce side-by-side comparisons of the parameter estimates and
#' diagnostic plots.
#' 
#' There are two distinct ways the \code{fit.models} function can be used.
#' 
#' The first is to fit the same model using different estimators.  In this
#' case, \code{model.list} should be a character vector or a list where each
#' element is the name of a modeling function and the remaining arguments (in
#' \dots) are the common arguments to the functions in \code{model.list}.
#' For example, the following command fits robust and least squares linear
#' models to Brownlee's Stack Loss Plant Data. \preformatted{
#' fit.models(c("rlm", "lm"), stack.loss ~ ., data = stackloss)} The resulting
#' \code{fit.models} object is a list with the output of \preformatted{
#' rlm(stack.loss ~ ., data = stackloss)} in the first element and
#' \preformatted{ lm(stack.loss ~ ., data = stackloss)} in the second.  The
#' class attribute of the returned list is set (in this case) to \code{"lmfm"}
#' which is the \code{fit.models} class (fmclass) for comparing linear-model-like
#' fits.
#' 
#' The second use of fit.models is to combine fitted model objects.  In
#' this case, \code{fit.models} combines its arguments into a fit.models object
#' (a list where element \eqn{i} is occupied by argument \eqn{i} and sets the
#' class attribute to the appropriate \code{fit.models} class.
#' 
#' @param model.list a list or a character vector containing names of modeling
#' functions.  Only required when \code{fit.models} is being used to fit models
#' (rather than combine already fitted models into a \code{fit.models} object).
#' @param \dots see details.
#' @return The returned object is a list containing the fitted models.  The
#' class of the retuned object depends on the classes of the model objects it
#' contains.
#' @seealso \code{\link{fmclass.add.class}} for adding a class to an existing
#' fit.models class and \code{\link{fmclass.register}} to create a new
#' fit.models class.
#' @keywords models
#' @examples
#'   # First, use fit.models to fit robust and least squares linear
#'   # regression models to Brownlee's Stack Loss Plant Data.
#' 
#'   # Step 1: rlm (robust linear model) is in the MASS package.
#'   library(MASS)
#' 
#'   # Step 2: tell fit.models rlm can be compared to lm
#'   fmclass.add.class("lmfm", "rlm")
#' 
#'   fm1 <- fit.models(c("rlm", "lm"), stack.loss ~ ., data = stackloss)
#' 
#'   summary(fm1) #rlm does not provide p-values or Multiple R-squared
#' 
#' 
#'   # Second, use fit.models to combine fitted models into a
#'   # fit.models object.
#' 
#'   lm.complete <- lm(stack.loss ~ ., data = stackloss)
#'   lm.clean <- lm(stack.loss ~ ., data = stackloss, subset = 5:20)
#' 
#'   fm2 <- fit.models(lm.clean, lm.complete)
#' 
#'   summary(fm2)
#'   plot(fm2)
#' 
#' 
#'   # Name the models in the fit.models object.
#'   
#'   fm3 <- fit.models(c(Robust = "rlm", "Least Squares" = "lm"),
#'                     stack.loss ~ ., data = stackloss)
#' 
#'   fm4 <- fit.models(Clean = lm.clean, Complete = lm.complete)


#' @export
fit.models <- function(model.list, ...)
{
  fm.call <- match.call()
  fm.call$attributes <- NULL

  dots <- list(...)
  dots.names <- names(dots)

  if(is.null(dots.names))
    dots.names <- character(length(dots))

  supported.classes <- unlist(sapply(e$fmreg, function(u) u$classes))


  ## The only way model list can be missing is if all the arguments in the call
  ## are named. In this case, proceed as if dots is a collection of comparable fitted
  ## models.

  if(missing(model.list)) {
    model.list <- dots
    model.names <- dots.names
  }


  ## Otherwise model.list is either an unnamed (in the call) fitted model or
  ## a list of function names.

  else if(class(model.list)[1] %in% supported.classes) {
    model.list <- c(list(model.list), dots)
    model.names <- c("", dots.names)

    object.names <- as.character(fm.call[-1])
    empty.names <- (nchar(model.names) == 0)
    model.names[empty.names] <- object.names[empty.names]
  }

  else if(is.character(model.list) || class(model.list)[1] == "list") {
    model.list <- as.list(model.list)

    ## Little hack for robust package backward compatibility
    model.list[model.list == "cov"] <- "covMLE"

    n.models <- length(model.list)
    model.funs <- unlist(model.list)

    if(is.null(model.names <- names(model.list)))
      model.names <- model.funs

    model.call <- fm.call
    model.call$model.list <- NULL

    for(i in 1:n.models) {
      model.call[[1]] <- as.name(model.list[[i]])
      model.list[[i]] <- eval(model.call, sys.parent())
    }

    empty.names <- (nchar(model.names) == 0)
    model.names[empty.names] <- model.funs[empty.names]
  }

  else
    stop("impossible error: this should never happen!")

  if(any(nchar(model.names) == 0))
    stop("All models should be named")


  ## Each element of model.list should have a unique name.

  if(length(unames <- unique(model.names)) < length(model.list)) {
    for(n in unames) {
      idx <- (model.names == n)
      if(sum(idx) > 1)
        model.names[idx] <- paste(n, 1:sum(idx), sep = ".")
    }
  }

  names(model.list) <- model.names


  ## Now we should have a properly named list of fitted models.  Have to
  ## set the appropriate attributes.

  candidates <- lapply(e$fmreg, getElement, name = "classes")
  classes <- sapply(model.list, function(u) class(u)[1])

  ## First, the fm class must beable to compare all the classes.

  idx <- sapply(candidates, function(u) all(classes %in% u))
  candidates <- candidates[idx]

  if(!length(candidates)) {
    warning("fit.models cannot compare the provided models")
    return(invisible(model.list))
  }


  ## Try to choose the best comparable class.

  idx <- sapply(candidates, function(u) length(intersect(u, classes)))
  fmclass <- names(which(idx == max(idx)))[1]

  oldClass(model.list) <- c(fmclass, "fit.models")
  model.list
}

#' Comparison Screeplot
#' 
#' @description Overlaid screeplot for the models in a \dQuote{covfm} object.
#'
#' @param x a \code{"covfm"} object.
#' 
#' @param npcs a postive integer value specifying the number of components to be plotted.
#' 
#' @param strip a character string printed in the \dQuote{strip} at the top of the plot.
#' 
#' @param ... additional arguments are passed to \code{xyplot}.
#'
#' @return the \code{trellis} object is invisibly returned.
#' 
#' @export
screePlot.covfm <- function(x, npcs, strip = "", ...)
{
  n.models <- length(x)
  mod.names <- names(x)
  dots <- list(...)

  eval.extractor <- function(u)
    eigen(vcov(u), symmetric = TRUE, only.values = TRUE)$values
  evals <- lapply(x, eval.extractor)
  
  if(missing(npcs))
    npcs <- min(10, max(sapply(evals, length)))

  for(i in 1:n.models)
    if(length(evals[[i]]) > npcs)
      evals[[i]] <- evals[[i]][1:npcs]

  n.evals <- sapply(evals, length)

  tdf <- data.frame(evals = unlist(evals),
                    index = unlist(lapply(n.evals, function(u) 1:u)),
                    mod = factor(rep(mod.names, n.evals), levels = mod.names))

  key <- simpleKey(text = mod.names, lines = TRUE, x = 0.95, y = 0.925, corner = c(1, 1), ...)

  if(!is.null(pch <- dots$pch) && length(pch) == n.models)
    key$points$pch <- pch

  x.scale <- list(x = list(at = 1:npcs,
                           labels = paste("Comp", 1:npcs, sep = ".")))

  p <- xyplot(evals ~ index | strip,
              data = tdf,
              groups = mod,
              strip = function(...) strip.default(..., style = 1),
              type = "b",
              key = key,
              scales = x.scale,
              ...)

  print(p)
  invisible(p)
}


utils::globalVariables(c("mod"))

#' Comparison Kernel Density Estimate Plot
#' 
#' Produces side-by-side kernel density estimate plots.
#' 
#' 
#' @param x a \code{fit.models} object.
#' @param fun a function to extract the desired quantity from \code{x}.
#' @param \dots additional arguments are passed to
#' \code{\link[lattice]{xyplot}}.
#' @return the \code{trellis} object is invisibly returned.
#' @keywords hplot


#' @importFrom lattice densityplot panel.densityplot panel.abline strip.default


#' @export
sideBySideKernelDensityPlot <- function(x, fun, ...)
{
  n.models <- length(x)
  mod.names <- names(x)

  y <- lapply(x, fun)
  n.y <- sapply(y, length)

  panel.special <- function(x, y, ...)
  {
    panel.densityplot(x, ...)
    panel.abline(v = 0, lty = 2)
    invisible()
  }

  mod <- factor(rep(mod.names, n.y), levels = mod.names)
  tdf <- data.frame(y = unlist(y), mod = mod)

  p <- densityplot(~ y | mod,
                   data = tdf,
                   n = 256,
                   bw = "SJ",
                   plot.points = FALSE,
                   ref = TRUE,
                   panel = panel.special,
                   strip = function(...) strip.default(..., style = 1),
                   layout = c(n.models, 1, 1),
                   ...)

  print(p)
  invisible(p)
}

#' @importFrom stats coef

#' @export
coef.lmfm <- function(object, ...)
{
  n.models <- length(object)
  mod.names <- names(object)
  coefs <- lapply(object, coef)
  coef.names <- unique(unlist(lapply(coefs, names)))

  coefmat <- matrix(NA, n.models, length(coef.names))
  dimnames(coefmat) <- list(mod.names, coef.names)
  for(i in 1:n.models)
    coefmat[i, names(coefs[[i]])] <- coefs[[i]]

  coefmat
}

.onLoad <- function(libname, pkgname)
{
  e$fmreg <- list()

  fmclass.register(fmclass = "lmfm",
                   classes = c("lm", "rlm"),
                   validation.function = NULL)
                                 
  fmclass.register(fmclass = "glmfm",
                   classes = c("glm"),
                   validation.function = NULL)

  fmclass.register(fmclass = "covfm",
                   classes = c("covClassic", "covRob"),
                   validation.function = NULL)

  invisible()
}

#' Plot Method
#'
#' @description Generic plot method for \dQuote{covfm} objects.
#'
#' @param x a \code{covfm} object.
#' 
#' @param which.plots either \code{"ask"} (character string) or an integer vector specifying
#'                    which plots to draw.  The plot options are (1) Mahalanobis Distance,
#'                    (2) Ellipses Matrix, (3) Screeplot (Eigenvalues of Covariance Estimate),
#'                    and (4) Distance - Distance Plot.
#'
#' @param ... additional arguments are passed to the plot subfunctions.
#'
#' @return \code{x} is returned invisibly.
#' 
#' @export
plot.covfm <- function(x, which.plots = 1:4, ...)
{
  n.models <- length(x)
  mod.names <- names(x)

  if(length(which.plots) == 1 && casefold(which.plots) == "all")
    which.plots <- 1:4

  choices <- c("Mahalanobis Distance",
               "Ellipses Plot",
               "Screeplot")

  if(n.models == 2) {
    choices <- c(choices, "Distance - Distance Plot")
    all.plots <- 1:4
  }
  else
    all.plots <- 1:3

  if(length(which.plots) == 0)
    return(invisible(x))

  if(is.numeric(which.plots)) {
    which.plots <- intersect(which.plots, all.plots)

    if(length(which.plots) > 1) {
      par.ask <- par(ask = TRUE)
      on.exit(par(ask = par.ask))
    }
    which.plots <- c(which.plots, 0)

    ask <- FALSE
  }

  else
    ask <- TRUE

  repeat {
    if(ask)
      which.plots <- menu(choices, title = "\nMake plot selections (or 0 to exit):")

    if(!length(which.plots))
      stop(paste("Invalid choice of plot in \'which.plots\'"))

    for(pick in (1 + which.plots)) {
      switch(pick,
        return(invisible(x)),

        distancePlot.covfm(x,
                          xlab = "Index",
                          ylab = "Mahalanobis Distance",
                          pch = 16,
                          ...),

        ellipsesPlot.covfm(x, ...),
        
        screePlot.covfm(x,
                       xlab = "Principal Component",
                       ylab = "Variance",
                       main = "Screeplot",
                       ...),

        {
          strip <- paste(mod.names[1], "Distance vs.", mod.names[2], "Distance")
          ddPlot.covfm(x,
                      strip = strip,
                      xlab = paste(mod.names[2], "Distance"),
                      ylab = paste(mod.names[1], "Distance"),
                      main = "Distance-Distance Plot",
                      pch = 16,
                      ...)
        }
      )
    }
  }
  invisible(x)
}

#' Scatter Plot with Overlaid Fits
#' 
#' Produces a scatter plot of the data with overlaid fits.
#' 
#' 
#' @param x a \code{fit.models} object.
#' @param lwd.reg a vector with length equal to the number of models in
#' \code{x} specifying the line widths used in the plot.
#' @param col.reg a vector with length equal to the number of models in
#' \code{x} specifying the line colors used in the plot.
#' @param \dots additional arguments are passed to
#' \code{\link[lattice]{xyplot}}.
#' @return the \code{trellis} object is invisibly returned.
#' @keywords hplot


#' @importFrom lattice xyplot panel.xyplot panel.abline simpleKey
#' @importFrom stats as.formula formula


#' @export
overlaidSimpleRegressionPlot <- function(x, lwd.reg, col.reg, ...) 
{
  n.models <- length(x)
  mod.names <- names(x)

  mf <- sapply(x, function(u) !is.null(u$model))

  if(!any(mf))
    stop("none of the fitted models in ", sQuote(deparse(substitute(x))),
         "contain a model frame component")

  mf <- x[[(1:n.models)[mf][1]]]$model

  if(ncol(mf) != 2)
    stop(sQuote("x"), " is not a simple linear regression model")

  var.names <- attributes(mf)$names
  frm <- as.formula(paste(paste(var.names, collapse = " ~ "), " | \"\""))

  if(missing(lwd.reg))
    lwd.reg <- 1:n.models

  if(missing(col.reg))
    col.reg <- 1:n.models

  panel.special <- function(x, y, object, lwd.reg, col.reg, ...)
  {
    panel.xyplot(x, y, ...)

    for(i in 1:length(object)) {
#      if(length(grep("Rob", object[[i]]$call))) {
#        a <- object[[i]]$yc * object[[i]]$scale
#        panel.abline(coef(object[[i]]) + c(-a, 0), lty = 2, col = col[i])
#        panel.abline(coef(object[[i]]), lwd = lwd[i], col = col[i])
#        panel.abline(coef(object[[i]]) + c(a, 0), lty = 2, col = col[i])
#      }
#      else {
        panel.abline(coef(object[[i]]), lwd = lwd.reg[i], col = col.reg[i])
#      }
    }
    invisible()
  }

  if(coef(x[[1]])[2] > 0)
    corner <- c(0.05, 0.95)
  else
    corner <- c(0.95, 0.95)

  key <- simpleKey(corner = corner,
                   text = mod.names,
                   points = FALSE,
                   lines = TRUE)
  key$lines$col <- col.reg
  key$lines$lwd <- lwd.reg

  mod <- rep(format(formula(x[[1]])), dim(mf)[1])

  p <- xyplot(mf[[1]] ~ mf[[2]] | mod,
              panel = panel.special,
              object = x,
              col.reg = col.reg,
              lwd.reg = lwd.reg,
              xlab = var.names[2],
              ylab = var.names[1],
              key = key,
              ...)

  print(p)
  invisible(p)
}

#' Comparison Scatter Plot
#' 
#' Produces a side-by-side scatter plot.
#' 
#' 
#' @param object a \code{fit.models} object.
#' @param x.fun a function to extract the x-axis quantity.
#' @param y.fun a function to extract the y-axis quantity.
#' @param \dots additional arguments.
#' @return the \code{trellis} object is invisibly returned.
#' @keywords hplot


#' @importFrom lattice xyplot panel.xyplot strip.default


#' @export
sideBySideScatterPlot <- function(object, x.fun, y.fun, ...)
{
  n.models <- length(object)
  mod.names <- names(object)

  x <- lapply(object, x.fun)
  n.x <- sapply(x, length)
  y <- lapply(object, y.fun)

  mod <- factor(rep(mod.names, n.x), levels = mod.names)
  tdf <- data.frame(x = unlist(x), y = unlist(y), mod = mod)

  panel.special <- function(x, y, ...) {
    panel.xyplot(x, y, ...)
    panel.addons(x, y)
    invisible()
  }

  p <- xyplot(y ~ x | mod,
              data = tdf,
              panel = panel.special,
              strip = function(...) strip.default(..., style = 1),
              layout = c(n.models, 1, 1),
              ...)

  print(p)
  invisible(p)
}

options(warn = 2)

library(fit.models)
library(MASS)

################################################################################
# Test models with differenct formulas
################################################################################

counts <- c(18, 17, 15, 20, 10, 20, 25, 13, 12)
outcome <- gl(3, 1, 9)
treatment <- c(gl(3,2), gl(3,1))
d.AD <- data.frame(treatment, outcome, counts)

trt <- glm(counts ~ treatment, family = poisson())
out <- glm(counts ~ outcome, family = poisson())

fm1 <- fit.models(trt, out)
print(fm1)
print(fm1.sum <- summary(fm1, correlation = TRUE))

pdf("fm1.pdf")
plot(fm1, 1)
plot(fm1, 2)
plot(fm1, 3)
plot(fm1, 4)
plot(fm1, 5)
plot(fm1, 6)
dev.off()

rm(fm1, fm1.sum)
file.remove("fm1.pdf")


################################################################################
# Test models with different formulas
################################################################################


## an example with offsets from Venables & Ripley (2002, p.189)
utils::data(anorexia, package = "MASS")

offset <- glm(Postwt ~ Prewt + Treat + offset(Prewt), family = gaussian,
              data = anorexia)

no.offset <- glm(Postwt - Prewt ~ Treat, family = gaussian, data = anorexia)

fm2 <- fit.models(offset, no.offset)
print(fm2)
print(fm2.sum <- summary(fm2, correlation = TRUE))

pdf("fm2.pdf")
plot(fm2, 1)
plot(fm2, 2)
plot(fm2, 3)
plot(fm2, 4)
plot(fm2, 5)
plot(fm2, 6)
dev.off()

rm(fm2, fm2.sum)
file.remove("fm2.pdf")


################################################################################
# Test models with different formulas and subsets
################################################################################

m1 <- glm(Postwt ~ Prewt + Treat, family = gaussian,
          data = subset(anorexia, Prewt > 75))

m2 <- glm(Postwt ~ Treat, family = gaussian, data = anorexia)

fm3 <- fit.models(m1, m2)
print(fm3)
print(fm3.sum <- summary(fm3, correlation = TRUE))

pdf("fm3.pdf")
plot(fm3, 1)
plot(fm3, 2)
plot(fm3, 3)
plot(fm3, 4)
plot(fm3, 5)
plot(fm3, 6)
dev.off()

rm(fm3, fm3.sum)
file.remove("fm3.pdf")


################################################################################

#' Comparison Diagnostic Plots for Linear Regression Models
#' 
#' Produces a set of comparison diagnostic plots.  The plot options are
#' \enumerate{
#'   \item Normal QQ Plot of Residuals,
#'   \item Kernel Density Estimate of Residuals,
#'   \item Residuals vs. Mahalanobis Distance,
#'   \item Residuals vs. Fitted Values,
#'   \item Sqrt Residuals vs. Fitted Values,
#'   \item Response vs. Fitted Values,
#'   \item Residuals vs. Index (Time),
#'   \item Overlaid Normal QQ Plot of Residuals,
#'   \item Overlaid Kernel Density Estimate of Residuals,
#'   \item Scatter Plot with Overlaid Fits (for simple linear regression models).
#'   }
#' 
#' @param x an \code{lmfm} object.
#' @param which.plots either \code{"ask"} (character string) or an integer
#' vector specifying which plots to draw. In the later case, the plot numbers
#' are given above.
#' @param \dots additional parameters are ignored.
#' @return \code{x} is invisibly returned.
#' @section Side Effects: The selected plots are drawn on a graphics device.
#' @seealso See \code{\link{sideBySideQQPlot}} for 2,
#' \code{\link{sideBySideKernelDensityPlot}} for 3,
#' \code{\link{sideBySideIndexPlot}} for 8,
#' \code{\link{overlaidQQPlot}} for 9,
#' \code{\link{overlaidKernelDensityPlot}} for 10,
#' \code{\link{overlaidSimpleRegressionPlot}} for 11, and
#' \code{\link{sideBySideScatterPlot}} for the others.
#' @keywords hplot methods
#' @examples
#' data(stackloss)
#' stack.lm <- lm(stack.loss ~ ., data = stackloss)
#' stack.clean <- lm(stack.loss ~ ., data = stackloss, subset = 5:20)
#' fm <- fit.models(stack.clean, stack.lm)
#' plot(fm)


# The \emph{modified residuals} are defined to be
# 
# \deqn{r_{i} = \frac{e_{i}}{\sqrt{1 - h_{i}}}}
# 
# where \eqn{h_{i} = H_{ii}} is the \eqn{i^{th}} diagonal element of the hat
# matrix.  The modified residuals are identically distributed with variance
# \eqn{\sigma^{2}}.  The modified residuals are used instead of the
# standardized residuals (which are identically distributed with variance 1)
# so that the comparison plots emphasize differences in the variance
# estimates.


#' @importFrom stats residuals fitted model.response model.frame
#' @importFrom graphics par
#' @importFrom utils menu


#' @export 
plot.lmfm <- function(x, which.plots = 1:10, ...)
{
  if(length(which.plots) == 1 && casefold(which.plots) == "all")
    which.plots <- 1:10

  choices <- c("Normal QQ Plot of Residuals", 
               "Kernel Density Estimate of Residuals",
               "Residuals vs. Mahalanobis Distance",
               "Residuals vs. Fitted Values", 
               "Sqrt Residuals vs. Fitted Values", 
               "Response vs. Fitted Values", 
               "Residuals vs. Index (Time)", 
               "Overlaid Normal QQ Plot of Residuals", 
               "Overlaid Kernel Density Estimate of Residuals")

  is.simple.reg <- function(m)
    all(attr(m$terms, "dataClasses") == "numeric") && dim(model.frame(m))[2] == 2

  if(all(sapply(x, is.simple.reg))) {
    choices <- c(choices, "Scatter Plot with Overlaid Fit(s)")
    all.plots <- 1:10
  }
  else
    all.plots <- 1:9

  if(length(which.plots) == 0)
    return(invisible(x))

  if(is.numeric(which.plots)) {
    which.plots <- intersect(which.plots, all.plots)

    if(length(which.plots) > 1) {
      par.ask <- par(ask = TRUE)
      on.exit(par(ask = par.ask))
    }
    which.plots <- c(which.plots, 0)

    ask <- FALSE
  }

  else
    ask <- TRUE

  n.models <- length(x)
  if(n.models <= 3)
    colors <- c("black", "blue", "purple")[1:n.models]
  else
    colors <- 1:n.models

  repeat {
    if(ask)
      which.plots <- menu(choices, title = "\nMake plot selections (or 0 to exit):")

    for(pick in (1 + which.plots)) {
      switch(pick,
        return(invisible(x)),

        sideBySideQQPlot(x,
                         fun = residuals,
                         main = expression(plain("Normal QQ Plot of Residuals")),
                         xlab = expression(plain("Standard Normal Quantiles")),
                         ylab = expression(plain("Empirical Quantiles of Residuals")),
                         pch = 16),

        sideBySideKernelDensityPlot(x,
                                    fun = residuals,
                                    main = expression(plain("Kernel Density Estimate of Residuals")),
                                    xlab = expression(plain("Residuals")),
                                    ylab = expression(plain("Density"))),

        sideBySideScatterPlot(x,
                              x.fun = function(u) sqrt(designMD(u)),
                              y.fun = residuals,
                              xlab = expression(plain("Mahalanobis Distance")),
                              ylab = expression(plain("Residuals")),
                              main = expression(plain("Residuals vs. Mahalanobis Distance")),
                              pch = 16),

        sideBySideScatterPlot(x,
                              x.fun = fitted,
                              y.fun = residuals,
                              main = expression(plain("Residuals vs. Fitted Values")),
                              xlab = expression(plain("Fitted Values")),
                              ylab = expression(plain("Residuals")),
                              pch = 16),

        sideBySideScatterPlot(x,
                              x.fun = fitted,
                              y.fun = function(u) sqrt(abs(residuals(u))),
                              main = expression(paste(sqrt(abs(plain("Residuals"))), plain(" vs. Fitted Values"))),
                              xlab = expression(plain("Fitted Values")),
                              ylab = expression(sqrt(abs(plain("Residuals")))),
                              pch = 16),

        sideBySideScatterPlot(x,
                              x.fun = fitted,
                              y.fun = function(u) model.response(model.frame(u)),
                              main = expression(plain("Response vs. Fitted Values")),
                              xlab = expression(plain("Fitted Values")),
                              ylab = expression(plain("Response")),
                              pch = 16),

        sideBySideIndexPlot(x,
                            fun = residuals,
                            main = expression(plain("Residuals vs. Index (Time)")),
                            xlab = expression(plain("Index (Time)")),
                            ylab = expression(plain("Residuals")),
                            pch = 16),

        overlaidQQPlot(x,
                       fun = residuals,
                       main = expression(plain("Normal QQ Plot of Residuals")),
                       xlab = expression(plain("Standard Normal Quantiles")),
                       ylab = expression(plain("Empirical Quantiles of Residuals")),
                       pch = 16),

        overlaidKernelDensityPlot(x,
                                  fun = residuals,
                                  main = expression(plain("Kernel Density Estimate of Residuals")),
                                  xlab = expression(plain("Residuals")),
                                  ylab = expression(plain("Density"))),

        overlaidSimpleRegressionPlot(x,
                                     main = expression(plain("Scatter Plot with Overlaid Fits")),
                                     lwd.reg = n.models:1,
                                     col.reg = colors,
                                     pch = 16)
      )
    }
  }

  invisible(x)
}

#' Comparison Summaries for Generalized Linear Models
#' 
#' Compute a summary of each model in a \code{glmfm} object.
#' 
#' 
#' @param object a glmfm object.
#' @param correlation a logical value. If \code{TRUE}, correlation matrices of
#' the coefficient estimates are included in each summary.
#' @param \dots additional arguments required by the generic
#' \code{\link{summary}} function.
#' @return a list with class summary.glmfm whose elements are summaries of each
#' model in \code{object}.
#' @keywords regression methods
#' @examples
#' 
#' # From ?glm:
#' # A Gamma example, from McCullagh & Nelder (1989, pp. 300-2)
#' 
#' clotting <- data.frame(
#'     u = c(5,10,15,20,30,40,60,80,100),
#'     lot1 = c(118,58,42,35,27,25,21,19,18),
#'     lot2 = c(69,35,26,21,18,16,13,12,12))
#' 
#' lot1 <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
#' lot2 <- glm(lot2 ~ log(u), data = clotting, family = Gamma)
#' 
#' fm <- fit.models(lot1, lot2)
#' summary(fm)


#' @export
summary.glmfm <- function(object, correlation = FALSE, ...)
{
  object <- lapply(object, summary, correlation = correlation, ...)
  oldClass(object) <- "summary.glmfm"
  object
}

#' Comparison Index (Time) Plot
#' 
#' Produces side-by-side index (time) plots.
#' 
#' 
#' @param x a \code{fit.models} object.
#' @param fun a function to extract the desired quantity from \code{x}.
#' @param level a numeric value between 0 and 1 specifying the confidence level
#' used to draw the threshold in the plot.
#' @param id.n a non-negative integer value specifying the number of extreme
#' points to identify.
#' @param \dots any additional arguments are passed to
#' \code{\link[lattice]{xyplot}}.
#' @return the \code{trellis} object is invisibly returned.
#' @keywords hplot


#' @importFrom lattice xyplot panel.xyplot panel.text panel.abline strip.default


#' @export
sideBySideIndexPlot <- function(x, fun, level = 0.95, id.n = 3, ...)
{
  n.models <- length(x)
  mod.names <- names(x)

  y <- lapply(x, fun)
  n.y <- sapply(y, length)

  s <- numeric(n.models)
  for(i in 1:n.models) {
    if(is.null(x[[i]]$scale))
      s[i] <- sqrt(sum(y[[i]]^2) / x[[i]]$df.residual)
    else
      s[i] <- x[[i]]$scale
  }

  indices <- lapply(x, function(u) attributes(model.frame(u))$row.names)

  if(all(sapply(indices, class) == "integer")) {
    newx <- min(unlist(indices)):max(unlist(indices))
    for(i in 1:n.models) {
      newy <- rep(as.numeric(NA), length(newx))
      newy[indices[[i]]] <- y[[i]]
      indices[[i]] <- newx
      y[[i]] <- newy
    }
  }

  else
    indices <- lapply(indices, function(u) 1:length(u))

  n.y <- sapply(y, length)

  for(i in 1:n.models) {
    y[[i]] <- c(s[i], y[[i]])
    indices[[i]] <- c(NA, indices[[i]])
  }

  y.range <- max(c(abs(range(unlist(y)))), s * qnorm(0.5 + level / 2.0))
  y.range <- 1.05 * c(-y.range, y.range)

  panel.special <- function(x, y, level = level, id.n = 3, ...)
  {
    s <- y[1]
    x <- x[-1]
    y <- y[-1]
    type <- ifelse(length(y) > 60, "l", "b")
    panel.xyplot(x, y, type = type, ...)
    s <- s * qnorm(0.5 + level / 2.0)
    if(id.n > 0) {
      n <- length(y)
      out <- order(abs(y))[(n - id.n + 1):n]
      out <- out[abs(y)[out] >= s]
      panel.text(x[out], y[out], paste(" ", x[out], sep = ""), adj = 0, ...)
    }
    panel.abline(h = s, lty = 2)
    panel.abline(h = -s, lty = 2)
    invisible()
  }

  mod <- factor(rep(mod.names, 1 + n.y), levels = mod.names)
  tdf <- data.frame(t = unlist(indices), y = unlist(y), mod = mod)

  p <- xyplot(y ~ t | mod,
              data = tdf,
              ylim = y.range,
              panel = panel.special,
              level = level,
              id.n = id.n,
              strip = function(...) strip.default(..., style = 1),
              layout = c(n.models, 1, 1),
              ...)

  print(p)
  invisible(p)
}

#' Side-by-Side Mahalanobis Distance Plot
#' 
#' @description Produces side-by-side plots of Mahalanobis distance computed using the
#'              location and covariance matrix estimates contained in each element of a
#'              \code{covfm} object.
#'
#' @param x a \code{"covfm"} object.
#' 
#' @param level a single numeric value between 0 and 1 giving the chi-squared percent
#'              point used to compute the outlyingness threshold.
#'              
#' @param id.n a single nonnegative integer specifying the number of extreme points to
#'             label in the plot.
#'             
#' @param \dots additional arguments are passed to \code{xyplot}.
#'
#' @return the \code{trellis} object is invisibly returned.
#'
#' @export
distancePlot.covfm <- function(x, level = 0.95, id.n = 3, ...)
{
  n.models <- length(x)
  mod.names <- names(x)

  dists <- lapply(x, function(u) u$dist)
  n <- sapply(dists, length)
  p <- sapply(x, function(u) nrow(u$cov))

  thresh <- qchisq(level, df = p)

  for(i in 1:n.models)
    dists[[i]] <- c(thresh[i], dists[[i]])

  panel.special <- function(x, y, id.n, ...) {
    x <- x[-1]
    vt <- y[1]
    y <- y[-1]
    n <- length(y)
    out <- which(y > vt)
    id.n <- min(id.n, length(out))

    panel.xyplot(x, y, ...)

    if(id.n > 0) {
      out <- order(y)[(n-id.n+1):n]
      panel.text(x[out], y[out], paste(" ", out, sep = ""), adj = 0)
    }

    panel.abline(h = vt, lty = 2)
    invisible()
  }

  mod <- factor(rep(mod.names, n+1), levels = mod.names)

  tdf <- data.frame(dists = sqrt(unlist(dists)),
                    index = unlist(lapply(n, function(u) 0:u)),
                    mod = mod)

  p <- xyplot(dists ~ index | mod,
              data = tdf,
              panel = panel.special,
              strip = function(...) strip.default(..., style = 1),
              layout = c(n.models, 1, 1),
              id.n = id.n,
              ...)

  print(p)
  invisible(p)
}

#' @export
print.covfm <- function(x, corr = FALSE, digits = max(3, getOption("digits") - 3), ...)
{
  n.models <- length(x)
  mod.names <- names(x)
  acc <- if(corr) vcor else vcov
  
  calls <- lapply(x, display.call)

  cat("\nCalls: \n")
  for(i in names(calls))
    cat(paste0(i, ": ", calls[[i]], "\n"))

  p <- nrow(acc(x[[1]]))
  i1 <- rep(seq(p), times = p)
  i2 <- rep(seq(p), each = p)

  cov.index <- paste("[", paste(i1, i2, sep = ","), "]", sep = "")
  cov.index <- matrix(cov.index, p, p)
  cov.index <- cov.index[row(cov.index) >= col(cov.index)]

  cov.unique <- t(sapply(x, function(u) (v <- acc(u))[row(v) >= col(v)]))
  dimnames(cov.unique) <- list(mod.names, cov.index)

  cat("\nComparison of Covariance/Correlation Estimates:\n")
  cat(" (unique covariance terms) \n")
  print(cov.unique, digits = digits, ...)

  loc <- t(sapply(x, center))
  loc.names <- names(center(x[[1]]))
  dimnames(loc) <- list(mod.names, loc.names)

  cat("\nComparison of center Estimates: \n")
  print(loc, digits = digits, ...)

  invisible(x)
}

#' @importFrom stats coef printCoefmat

#' @export
print.summary.glmfm <- function(x, digits = max(3, getOption("digits") - 3),
                                         signif.stars = getOption("show.signif.stars"),
                                         ...)
{
  n.models <- length(x)
  mod.names <- names(x)
  fancy.names <- format(paste(mod.names, ":", sep = ""), justify = "right")

  cat("\nCalls:\n")
  for(i in 1:n.models) {
    cat(fancy.names[i], " ", sep = "")
    print(x[[i]]$call, ...)
  }

  dev.res <- lapply(x, function(u) u$deviance.resid)
  has.dev.res <- which(!sapply(dev.res, is.null))

  if(length(has.dev.res)) {
    dev.res <- t(sapply(dev.res[has.dev.res], quantile))
    dimnames(dev.res) <- list(fancy.names[has.dev.res],
                              c("Min", "1Q", "Median", "3Q", "Max"))
    cat("\nDeviance Residuals:\n")
    print(dev.res, digits = digits, ...)
  }

  coefs <- lapply(x, coef)
  cnames <- unique(unlist(lapply(coefs, rownames)))
  n.coefs <- length(cnames)
  coefmat <- matrix(NA, n.coefs * (n.models + 1) - 1, 4)
  r.names <- rep(c("", fancy.names), n.coefs)[-1]
  idx <- seq(from = 1, by = n.models + 1, length.out = n.coefs)
  r.names[idx] <- paste(paste(cnames, ":", sep = ""), r.names[idx])
  rownames(coefmat) <- format(r.names, justify = "right")
  colnames(coefmat) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")

  for(i in 1:n.models)
    coefmat[idx + (i-1), ][cnames %in% row.names(coefs[[i]]), ] <- coefs[[i]]

  cat("\nCoefficients:\n")
  printCoefmat(coefmat, digits = digits, signif.stars = signif.stars,
               na.print = "", ...)

  null.devs <- sapply(x, function(u) u$null.deviance)
  has.null.dev <- which(!sapply(null.devs, is.null))
  null.df <- sapply(x, function(u) u$df.null)
  null.devs <- format(null.devs, digits = digits, ...)
  null.df <- format(null.df, digits = digits, ...)

  if(length(has.null.dev)) {
    cat("\nNull deviance:\n")
    for(i in has.null.dev)
      cat(" ", fancy.names[i], null.devs[i], "on", null.df[i],
          "degrees of freedom\n")
  }

  devs <- sapply(x, function(u) u$deviance)
  has.dev <- which(!sapply(devs, is.null))
  df <- sapply(x, function(u) u$df[2])
  devs <- format(devs, digits = digits, ...)
  df <- format(df, digits = digits, ...)

  if(length(has.dev)) {
    cat("\nResidual deviance:\n")
    for(i in has.null.dev)
      cat(" ", fancy.names[i], devs[i], "on", df[i], "degrees of freedom\n")
  }

  cat("\n")

  correlations <- lapply(x, function(u) u$correlation)
  if(all(!sapply(correlations, is.null))) {    
    if(any(sapply(correlations, NCOL) > 1)) {
      cat("Correlations:\n")
      for(i in 1:n.models) {
        if((p <- NCOL(correlations[[i]])) > 1) {        
          correl <- format(round(correlations[[i]], 2), nsmall = 2,
                           digits = digits, ...)
          correl[!lower.tri(correl)] <- ""
          cat(mod.names[i], ":\n", sep = "")
          print(correl[-1, -p, drop = FALSE], quote = FALSE, ...)
        }
        cat("\n")
      }
    }
  }

  invisible(x)
}

#' Summary Method
#'
#' @description Generic summary method for \dQuote{covfm} objects.
#'
#' @param object a \dQuote{covfm} object.
#' 
#' @param corr a logical value passed as an attribute to the \code{print}
#'             method. When \code{TRUE}, correlations are compared in the
#'             textual output.
#' 
#' @param \dots additional arguments for the summary method.
#'
##' @export
summary.covfm <- function(object, corr = FALSE, ...)
{
  object <- lapply(object, summary, ...)
  attr(object, "corr") <- corr
  oldClass(object) <- "summary.covfm"
  object
}

#' Compaison QQ Plot
#' 
#' Produces side-by-side QQ plots.  An optional simulated confidence envelope
#' can be included in each plot.
#' 
#' 
#' @param x a \code{fit.models} object.
#' @param fun a function to extract the desired quantity from \code{x}.
#' @param envelope a logical value.  If \code{TRUE} a \code{level} confidence
#' envelope is simulated for each QQ plot.
#' @param half.normal a logical value.  If \code{TRUE} the plot is drawn using
#' the absolute values.
#' @param n.samples a positive integer value giving the number of samples to
#' compute in the simulation of the confidence envelope.
#' @param level a numeric value between 0 and 1 specifying the confidence level
#' for the envelope.
#' @param id.n a non-negative integer value specifying the number of extreme
#' points to identify.
#' @param qqline a logical value.  If \code{TRUE}, a QQ line is included in the
#' plot.
#' @param \dots additional arguments are passed to
#' \code{\link[lattice]{xyplot}}.
#' @return the \code{trellis} object is invisibly returned.
#' @keywords hplot


#' @importFrom lattice xyplot panel.xyplot panel.abline llines strip.default
#' @importFrom stats rnorm quantile qnorm qqnorm


#' @export
sideBySideQQPlot <- function(x, fun, envelope = TRUE, half.normal = FALSE,
                             n.samples = 250, level = .95, id.n = 3, qqline = TRUE,
                             ...)
{
  confidence.envelope <- function(n, sd = 1, n.samples = 250, level = 0.95,
                                  half.normal = FALSE)
  {
    lower <- upper <- matrix(0.0, n, n.models)

    alphaOver2 <- (1.0 - level) / 2.0
    probs <- c(alphaOver2, 1.0 - alphaOver2)

    env <- matrix(rnorm(n * n.samples, sd = sd), n.samples, n)

    if(half.normal)
      env <- abs(env)

    env <- apply(env, 1, sort)
    env <- apply(env, 1, quantile, probs = probs)

    list(lower = env[1, ], upper = env[2, ])
  }

  n.models <- length(x)
  mod.names <- names(x)

  res <- lapply(x, fun)
  n.res <- sapply(res, length)

  px <- py <- list()
  for(i in 1:n.models) {
    tmp <- qqnorm(res[[i]], plot.it = FALSE)
    px[[i]] <- tmp$x
    py[[i]] <- tmp$y
  }

  if(half.normal) {
    py <- lapply(py, abs)
    px <- lapply(n.res, function (u) .5 + (0:(u-1)) / (2*u))
    px <- lapply(px, qnorm)
    for(i in 1:n.models)
      px[[i]][order(py[[i]])] <- px[[i]]
  }

  if(envelope) {
    sigma.hats <- numeric(n.models)
    for(i in 1:n.models) {
      if(!is.null(x[[i]]$scale))
        sigma.hats[i] <- x[[i]]$scale
      else {
        x.sum <- summary(x[[i]])
        if(!is.null(x.sum$dispersion))
          sigma.hats[i] <- sqrt(x.sum$dispersion)
        else if(!is.null(x.sum$sigma))
          sigma.hats[i] <- x.sum$sigma
        else
          stop("unable to determine residual scale")
      }
    }

    env <- list()
    for(i in 1:n.models)
      env[[i]] <- confidence.envelope(n.res[i], sigma.hats[i], n.samples,
                                      level, half.normal)
    lower <- lapply(env, function(u) u$lower)
    upper <- lapply(env, function(u) u$upper)

    den.range <- c(min(unlist(py), unlist(lower)),
                   max(unlist(py), unlist(upper)))
  }

  else
    den.range <- c(min(unlist(py)), max(unlist(py)))

  if(envelope && half.normal) {
    mod <- factor(rep(rep(mod.names, n.res), 3), levels = mod.names)

    tdf <- data.frame(py = c(unlist(py),
                             unlist(lower),
                             unlist(upper)),
                      px = rep(unlist(px), 3),
                      mod = mod)

    panel.special <- function(x, y, id.n, qqline, ...) {
      dat.idx <- 1:(length(x)/3)

      panel.xyplot(x[dat.idx], y[dat.idx], ...)

      if(qqline) {
        u <- quantile(x[!is.na(x)], c(0.25, 0.75))
        v <- quantile(y[!is.na(y)], c(0.25, 0.75))
        slope <- diff(v) / diff(u)
        int <- v[1] - slope * u[1]
        panel.abline(int, slope, ...)
      }

      panel.addons(x[dat.idx], y[dat.idx], id.n = id.n)

      dat.idx <- ((length(x)/3)+1):(2*length(x)/3)

      llines(sort(x[dat.idx]), sort(y[dat.idx]), col.line = "black", lty = 2)

      dat.idx <- (2*(length(x)/3)+1):(length(x))

      llines(sort(x[dat.idx]), sort(y[dat.idx]), col.line = "black", lty = 2)

      invisible()
    }
  }

  else if(envelope) {
    mod <- factor(rep(rep(mod.names, n.res), 3), levels = mod.names)

    tdf <- data.frame(py = c(unlist(py),
                             unlist(lower),
                             unlist(upper)),
                      px = rep(unlist(px), 3),
                      mod = mod)

    panel.special <- function(x, y, id.n, qqline, ...) {
      dat.idx <- 1:(length(x)/3)

      panel.xyplot(x[dat.idx], y[dat.idx], ...)

      panel.addons(x[dat.idx], y[dat.idx], id.n = id.n)

      if(qqline) {
        u <- quantile(x[!is.na(x)], c(0.25, 0.75))
        v <- quantile(y[!is.na(y)], c(0.25, 0.75))
        slope <- diff(v) / diff(u)
        int <- v[1] - slope * u[1]
        panel.abline(int, slope, ...)
      }

      dat.idx <- ((length(x)/3)+1):(2*length(x)/3)

      llines(sort(x[dat.idx]), sort(y[dat.idx]), col.line = "black", lty = 2)

      dat.idx <- (2*(length(x)/3)+1):(length(x))

      llines(sort(x[dat.idx]), sort(y[dat.idx]), col.line = "black", lty = 2)

      invisible()
    }
  }

  else {
    mod <- factor(rep(mod.names, n.res), levels = mod.names)
    tdf <- data.frame(px = unlist(px), py = unlist(py), mod = mod)

    panel.special <- function(x, y, id.n, qqline, ...) {
      panel.xyplot(x, y, ...)
      panel.addons(x, y, id.n = id.n)

      if(qqline) {
        u <- quantile(x[!is.na(x)], c(0.25, 0.75))
        v <- quantile(y[!is.na(y)], c(0.25, 0.75))
        slope <- diff(v) / diff(u)
        int <- v[1] - slope * u[1]
        panel.abline(int, slope, ...)
      }

      invisible()
    }
  }

  p <- xyplot(py ~ px | mod,
              data = tdf,
              id.n = id.n,
              qqline = qqline,
              panel = panel.special,
              strip = function(...) strip.default(..., style = 1),
              layout = c(n.models, 1, 1),
              ...)

  print(p)
  invisible(p)
}

#' Comparison Summaries for Linear Regression Models
#' 
#' Compute a summary of each model in an \code{lmfm} object.
#' 
#' 
#' @param object an \code{lmfm} object.
#' @param correlation a logical value.  If TRUE, the correlation matrices for
#' the coefficients are included in the summaries.
#' @param \dots additional arguments required by the generic
#' \code{\link{summary}} function.
#' @return a list with class summary.lmfm whose elements are summaries of each
#' model in \code{object}.
#' @keywords methods regression
#' @examples
#' 
#' data(stackloss)
#' m1 <- lm(stack.loss ~ ., data = stackloss)
#' m2 <- lm(stack.loss ~ . - Acid.Conc., data = stackloss)
#' 
#' fm <- fit.models(m1, m2)
#' print(fm.sum <- summary(fm))


#' @export
summary.lmfm <- function(object, correlation = FALSE, ...)
{
  object <- lapply(object, summary, correlation = correlation, ...)
  oldClass(object) <- "summary.lmfm"
  object
}

#' @export
print.lmfm <- function(x, digits = max(3, getOption("digits") - 3), ...)
{
  n.models <- length(x)
  fancy.names <- format(paste(names(x), ":", sep = ""), justify = "right")

  cat("\nCalls:\n")
  for(i in 1:n.models) {
    cat(fancy.names[i], " ", sep = "")
    print(x[[i]]$call, ...)
  }

  cat("\nCoefficients:\n")
  printCoefmat(coef(x), digits = digits, P.values = FALSE, na.print = "",
               signif.stars = FALSE, ...)
  cat("\n")

  invisible(x)
}

#' @importFrom lattice panel.loess panel.rug panel.text

panel.addons <- function(x, y, smooths = FALSE, rugplot = FALSE, id.n = 3)
{
  if(smooths)
    panel.loess(x, y, col = "red")

  if(rugplot)
    panel.rug(x, col = "black")

  if(id.n > 0) {
    n <- length(y)
    out <- order(abs(y))[(n - id.n + 1):n]
    panel.text(x[out], y[out], paste(" ", out, sep = ""), adj = 0)
  }

  invisible()
}

#' @importFrom stats vcov

#' @export
vcov.default <- function(object, ...)
{
  if(!is.null(object$cov) && is.matrix(object$cov))
    return(object$cov)
  
  NULL
}


#' Calculate Correlation Matrix for a Fitted Model Object
#'
#' @description Retrieve a correlation matrix estimate from a
#'              fitted model object. The default method uses
#'              \code{\link[stats]{cov2cor}} to scale the
#'              covariance matrix returned by
#'              \code{\link[stats]{vcov}}.
#'
#' @param object a fitted model object, typically.  Sometimes
#'               also a \code{summary()} object of such a fitted
#'               model.
#'
#' @param \dots additional arguments for method functions.
#' 
#' @export
vcor <- function(object, ...)
  UseMethod("vcor")



#' @export
vcor.default <- function(object, ...)
{
  #' @importFrom stats vcov
  if(!is.null(v <- vcov(object)))
    #' @importFrom stats cov2cor
    return(cov2cor(v))
  
  NULL
}


#' Calculate Location Estimate for a Fitted Model Object
#'
#' @description Returns the location estimated from a
#'              location/scatter-type fitted model object.
#'
#' @param object a fitted model object, typically.  Sometimes
#'               also a \code{summary()} object of such a fitted
#'               model.
#'
#' @param \dots additional arguments for method functions.
#' 
#' @export
center <- function(object, ...)
  UseMethod("center")


#' @export
center.default <- function(object, ...)
{
  if(!is.null(object$center) && is.vector(object$center))
    return(object$center)
  
  NULL
}


valid.covfm <- function(x)
  !is.null(vcov(x)) && !is.null(center(x) && all(dim(vcov(x)) == length(center(x))))

#' Comparison Diagnostic Plots for Generalized Linear Models
#' 
#' Produces a set of comparison diagnostic plots.  The plot options are
#' \enumerate{
#'   \item Deviance Residuals vs. Predicted Values,
#'   \item Response vs. Fitted Values,
#'   \item Normal QQ Plot of Pearson Residuals,
#'   \item Normal QQ Plot of Deviance Residuals,
#'   \item Pearson Residuals vs. Mahalanobis Distance,
#'   \item Sqrt Deviance Residuals vs. Predicted Values.
#' }
#' 
#' @param x a \code{glmfm} object.
#' @param which.plots either \code{"ask"} (character string) or an integer
#' vector specifying which plots to draw. In the later case, the plot numbers
#' are given above.
#' @param \dots other parameters to be passed through to plotting functions.
#' @return \code{x} is invisibly returned.
#' @section Side Effects: The selected plots are drawn on a graphics device.
#' @seealso \code{\link{sideBySideQQPlot}} for 4 and 5 and
#' \code{\link{sideBySideScatterPlot}} for the others.
#' @keywords hplot methods
#' @examples
#' # From ?glm:
#' # A Gamma example, from McCullagh & Nelder (1989, pp. 300-2)
#' 
#' clotting <- data.frame(
#'     u = c(5,10,15,20,30,40,60,80,100),
#'     lot1 = c(118,58,42,35,27,25,21,19,18),
#'     lot2 = c(69,35,26,21,18,16,13,12,12))
#' 
#' lot1 <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
#' lot2 <- glm(lot2 ~ log(u), data = clotting, family = Gamma)
#' 
#' fm <- fit.models(lot1, lot2)
#' plot(fm)
#' 


#' @importFrom stats predict residuals fitted model.response model.frame
#' @importFrom graphics par
#' @importFrom utils menu

#' @export
plot.glmfm <- function(x, which.plots = 1:6, ...)
{
  if(length(which.plots) == 1 && casefold(which.plots) == "all")
    which.plots <- 1:6

  choices <- c("Deviance Residuals vs. Predicted Values",
               "Response vs. Fitted Values",
               "Normal QQ Plot of Pearson Residuals",
               "Normal QQ Plot of Deviance Residuals",
               "Pearson Residuals vs. Mahalanobis Distance",
               "Sqrt Deviance Residuals vs. Predicted Values")

  all.plots <- 1:6

  if(length(which.plots) == 0)
    return(invisible(x))

  if(is.numeric(which.plots)) {
    which.plots <- intersect(which.plots, all.plots)

    if(length(which.plots) > 1) {
      par.ask <- par(ask = TRUE)
      on.exit(par(ask = par.ask))
    }
    which.plots <- c(which.plots, 0)

    ask <- FALSE
  }

  else
    ask <- TRUE

  repeat {
    if(ask)
      which.plots <- menu(choices, title = "\nMake plot selections (or 0 to exit):")

    if(!length(which.plots))
      stop(paste("Invalid choice of plot in \'which.plots\'"))

    for(pick in (1 + which.plots)) {
      switch(pick,
        return(invisible(x)),

        sideBySideScatterPlot(x,
                              x.fun = predict,
                              y.fun = function(u) residuals(u, type = "deviance"),
                              xlab = expression(plain("Predicted Values")),
                              ylab = expression(plain("Deviance Residuals")),
                              main = expression(plain("Deviance Residuals vs. Predicted Values")),
                              pch = 16),

        sideBySideScatterPlot(x,
                              x.fun = fitted,
                              y.fun = function(u) model.response(model.frame(u)),
                              xlab = expression(plain("Fitted Values")),
                              ylab = expression(plain("Response")),
                              main = expression(plain("Response vs. Fitted Values")),
                              pch = 16),

        sideBySideQQPlot(x,
                         fun = function(u) residuals(u, type = "pearson"),
                         xlab = expression(plain("Standard Normal Quantiles")),
                         ylab = expression(plain("Empirical Quantiles of Pearson Residuals")),
                         main = expression(plain("Normal QQ Plot of Pearson Residuals")),
                         envelope = FALSE,
                         pch = 16),

        sideBySideQQPlot(x,
                         fun = function(u) residuals(u, type = "deviance"),
                         xlab = expression(plain("Standard Normal Quantiles")),
                         ylab = expression(plain("Empirical Quantiles of Deviance Residuals")),
                         main = expression(plain("Normal QQ Plot of Deviance Residuals")),
                         envelope = FALSE,
                         pch = 16),

        sideBySideScatterPlot(x,
                              x.fun = function(u) sqrt(designMD(u)),
                              y.fun = function(v) residuals(v, type = "pearson"),
                              xlab = expression(plain("Mahalanobis Distance")),
                              ylab = expression(plain("Pearson Residuals")),
                              main = expression(plain("Pearson Residuals vs. Mahalanobis Distance")),
                              pch = 16),

        sideBySideScatterPlot(x,
                              x.fun = predict,
                              y.fun = function(u) sqrt(abs(residuals(u, type = "deviance"))),
                              xlab = expression(plain("Predicted Values")),
                              ylab = expression(sqrt(abs(plain("Deviance Residuals")))),
                              main = expression(paste(sqrt(abs(plain("Deviance Residuals"))), plain(" vs. Predicted Values"))),
                              pch = 16)
      )
    }
  }

  invisible(x)
}

#' Design Matrix Mahalanobis Distance
#' 
#' Returns the squared Mahalanobis distance of all rows in the design (model)
#' matrix \eqn{X} and the sample mean vector \eqn{\mu} of the columns
#' of \eqn{X} with respect to the sample covariance matrix \eqn{\Sigma}.
#' This is (for vector \eqn{x'} a row of \eqn{X}) defined as
#' \deqn{d^{2} = (x - \mu)' \Sigma^{-1} (x - \mu)}
#' where
#' \deqn{\mu = colMeans(X)}
#' and
#' \deqn{\Sigma = cov(X).}
#' 
#' @aliases designMD designMD.default
#' @param object a fitted model object with a \code{\link{model.matrix}} method.
#' @param \dots additional arguments are ignored.
#' @return a numeric vector containing the squared Mahalanobis distances.
#' @keywords regression methods
#' @examples
#' stack.lm <- lm(stack.loss ~ ., data = stackloss)
#' 
#' # Mahalanobis distance (not squared)
#' sqrt(designMD(stack.lm))

#' @importFrom stats mahalanobis model.matrix terms var


#' @export
designMD <- function(object, ...)
  UseMethod("designMD")


#' @export
designMD.default <- function(object, ...)
{
  tr <- terms(object)
  x <- model.matrix(object)
  
  if(attr(tr, "intercept") == 1)
    x <- x[, -1, drop = FALSE]
  
  stats::mahalanobis(x, colMeans(x), var(x))
}

#' @export
print.summary.covfm <- function(x, corr = FALSE, digits = max(3, getOption("digits") - 3),
                                print.distance = FALSE, ...)
{
  n.models <- length(x)
  mod.names <- names(x)

  if(!("corr" %in% names(match.call())[-1]) && !is.null(ac <- attr(x, "corr")))
    corr <- ac

  acc <- if(corr) vcor else vcov
  
  calls <- lapply(x, display.call)

  cat("\nCalls: \n")
  for(i in names(calls))
    cat(paste0(i, ": ", calls[[i]], "\n"))

  p <- dim(acc(x[[1]]))[1]
  i1 <- rep(seq(p), times = p)
  i2 <- rep(seq(p), each = p)

  cov.index <- paste("[", paste(i1, i2, sep = ","), "]", sep = "")
  cov.index <- matrix(cov.index, p, p)
  cov.index <- cov.index[row(cov.index) >= col(cov.index)]

  cov.unique <- t(sapply(x, function(u) (v <- acc(u))[row(v) >= col(v)]))
  dimnames(cov.unique) <- list(mod.names, cov.index)

  cat("\nComparison of Covariance/Correlation Estimates:\n")
  cat(" (unique correlation terms) \n")
  print(cov.unique, digits = digits, ...)

  loc <- t(sapply(x, center))
  loc.names <- names(center(x[[1]]))
  dimnames(loc) <- list(mod.names, loc.names)

  cat("\nComparison of center Estimates: \n")
  print(loc, digits = digits, ...)

  evals <- t(sapply(x, function(u) u$evals))
  eval.names <- names(x[[1]]$evals)
  dimnames(evals) <- list(mod.names, eval.names)

  cat("\nComparison of Eigenvalues: \n")
  print(evals, digits = digits, ...)

  have.dist <- sapply(x, function(u) !is.null(u$dist))
  if(print.distance && all(have.dist)) {
    dists <- t(sapply(x, function(u) u$dist))
    dimnames(dists) <- list(mod.names, names(x[[1]]$dist))
    cat("\nComparison of Mahalanobis Distances: \n")
    print(dists, digits = digits, ...)
  }

  invisible(x)
}

display.call <- function(object)
{
  if(is.null(the.call <- object$call)) {
    x <- class(object)[1]
    if(substring(x, 1, 8) == "summary.")
      x <- substring(x, 9)
    paste0(x, "(...)")
  }
  else
    format(the.call)
}

#' Overlaid Kernel Density Estimate Plot
#' 
#' Produces an overlaid kernel density plot.
#' 
#' 
#' @param x a \code{fit.models} object.
#' @param fun a function to extract the desired quantity from \code{x}.
#' @param \dots additional arguments are passed to
#' \code{densityplot}.
#' @return the \code{trellis} object is invisibly returned.
#' @keywords hplot


#' @importFrom lattice densityplot strip.default


#' @export
overlaidKernelDensityPlot <- function(x, fun, ...)
{
  n.models <- length(x)
  mod.names <- names(x)

  y <- lapply(x, fun)
  n.y <- sapply(y, length)
  mod <- factor(rep(mod.names, n.y), levels = mod.names)
  tdf <- data.frame(y = unlist(y), mod = mod)

  p <- densityplot(~ y | "",
                   groups = mod,
                   data = tdf,
                   n = 256,
                   bw = "SJ",
                   plot.points = FALSE,
                   strip = function(...) strip.default(..., style = 1),
                   auto.key = list(corner = c(0.95, 0.95)),
                   ...)

  print(p)
  invisible(p)
}

#' Ellipses Plot - Visual Correlation Matrix Comparison
#'
#' @description When there are 3 or more variables in the data, this function produces
#'              a matrix with ellipses drawn in the upper triangle.  The ellipse in cell
#'              \eqn{i,j} of the plot is drawn to be a contour of a standard bivariate
#'              normal with correlation \eqn{\rho_{ij}}.  One ellipse is drawn in each
#'              cell for each model in the \code{covfm} object.  When there are 2
#'              variables in the data, this function produces a scatter plot of the data
#'              with an overlaid 95\% confidence ellipse for each model in the
#'              \code{covfm} object.
#'
#' @param x a \code{"covfm"} object.
#' @param \dots additional arguments are ignored.
#'
#' @return x is invisibly returned.
#'
#' @importFrom graphics legend lines plot polygon strheight strwidth text
#'
#' @export
ellipsesPlot.covfm <- function(x, ...)
{
  n.models <- length(x)
  mod.names <- names(x)

  p <- dim(vcov(x[[1]]))[1]

  ## if p == 2 plot data with overlaid ellipse ##
  if(p == 2) {
    old.par <- par(pty = "s")
    on.exit(par(old.par))

    ellipse <- function(loc, A)
    {
      detA <- A[1, 1] * A[2, 2] - A[1, 2]^2
      dist <- sqrt(qchisq(0.95, 2))
      ylimit <- sqrt(A[2, 2]) * dist
      y <- seq(-ylimit, ylimit, 0.01 * ylimit)
      sqrt.discr <- detA/A[2,2]^2 * (A[2,2] * dist^2 - y^2)
      sqrt.discr[c(1, length(sqrt.discr))] <- 0.0
      sqrt.discr <- sqrt(sqrt.discr)
      b <- loc[1] + A[1, 2] / A[2, 2] * y
      x1 <- b - sqrt.discr
      x2 <- b + sqrt.discr
      y <- loc[2] + y
      rbind(cbind(x1, y), cbind(rev(x2), rev(y)))
    }

    z <- list()
    x.min <- Inf
    x.max <- -Inf
    y.min <- Inf
    y.max <- -Inf

    for(i in 1:n.models) {
      z[[i]] <- ellipse(center(x[[i]]), vcov(x[[i]]))
      x.min <- min(x.min, z[[i]][,1])
      x.max <- max(x.max, z[[i]][,1])
      y.min <- min(y.min, z[[i]][,2])
      y.max <- max(y.max, z[[i]][,2])
    }

    X <- try(eval(x[[1]]$call$data, envir = sys.parent(2)), silent = TRUE)
    if(!is.null(X) && !inherits(X, "try-error")) {
      X <- as.matrix(X)
      x.min <- min(x.min, X[,1])
      x.max <- max(x.max, X[,1])
      y.min <- min(y.min, X[,2])
      y.max <- max(y.max, X[,2])
    } else {
      X <- matrix(NA, 1, 2)
    }

    center <- c(mean(c(x.min, x.max)), mean(c(y.min, y.max)))

    s.range <- max(abs(c(center[1] - x.min, x.max - center[1],
                         center[2] - y.min, y.max - center[2])))

    if(n.models == 1)
      header <- "95% Tolerance Ellipse"
    else
      header <- "95% Tolerance Ellipses"

    plot(X,
         xlim = c(center[1] - s.range, center[1] + s.range),
         ylim = c(center[2] - s.range, center[2] + s.range),
         xlab = names(center(x[[1]]))[1],
         ylab = names(center(x[[1]]))[2],
         main = header,
         col = "lightgray",
         pch = 16,
         asp = 1)

    for(i in 1:length(z))
      polygon(z[[i]], density = 0, lty = i, col = i, lwd = 2)

    pos <- ifelse(vcov(x[[1]])[1,2] > 0, "topleft", "topright")
    legend(x = pos, legend = mod.names, col = 1:n.models, lty = 1:n.models,
           lwd = 2, bty = "n")
  }

  ## if p > 2 plot matrix of ellipses ##
  else {
    old.par <- par(mar = rep(0.1, 4), pty = "s")
    on.exit(par(old.par))

    labels <- dimnames(vcov(x[[1]]))[[1]]
    tl.margin <- max(strwidth(paste("WW", labels, sep = ""), units = "inches"))
    tl.margin <- 1.05*tl.margin / par()$fin[1]

    if(tl.margin < 0.05) {
      tl.margin <- 0.05
      cex.labels <- 1.0
    } else if(tl.margin > 0.2) {
      cex.labels <- 0.2 / tl.margin
      tl.margin <- 0.2
    } else {
      cex.labels <- 1.0
    }

    br.margin <- 0.025*(n.models - 1)

    plot(NA, NA,
         xlim = c(0.5 - p*tl.margin, 0.5 + (1 + br.margin)*p),
         ylim = c(0.5 - p*br.margin, (1 + tl.margin)*p + 0.5),
         type = "n",
         axes = FALSE,
         xlab = "",
         ylab = "")

    ht.corr <- (1.5 * n.models - 0.5) * strheight("8", units = "user")
    wt.corr <- strwidth("-0.00", units = "user")
    cex.corr <- min(c(0.75 / max(c(ht.corr, wt.corr)), 1.25))

    vert <- (1:n.models) * 1.6
    vert <- vert - mean(vert) + 0.5

    for(k in 1:n.models) {
      s <- sqrt(diag(vcov(x[[k]])))
      X <- vcov(x[[k]]) / (s %o% s)

      xc <- col(X)
      yc <- row(X)[p:1,]

      ut <- row(X) < col(X)
      lt <- row(X) > col(X)

      pts <- c(seq(0.0, 2*pi, length.out = 181), NA)
      xs <- sapply(X[ut], function(u, v) cos(v + acos(u)/2), v = pts)
      xs <- 0.475 * xs + rep(xc[ut], each = 182)
      ys <- sapply(X[ut], function(u, v) cos(v - acos(u)/2), v = pts)
      ys <- 0.475 * ys + rep(yc[ut], each = 182)

      polygon(x = as.vector(xs),
              y = as.vector(ys),
              density = 0,
              lwd = n.models - k + 1,
              col = k,
              lty = k)

      corr <- X[lt]
      corr[corr > 0.99 & corr < 1.00] <- 0.99
      corr[corr < -0.99 & corr > -1.00] <- -0.99
      corr <- format(round(corr, digits = 2))

      text(xc[lt] + 0.5*strwidth("0.00", cex = cex.corr),
           yc[lt],
           labels = corr,
           adj = c(1.0, vert[k]),
           col = k,
           cex = cex.corr)
    }

    lines(c(1, p), c(p, 1), col = "gray")

    text(1:p, p+0.5,
         paste("  ", labels, sep = ""),
         adj = 0,
         srt = 90,
         cex = cex.labels)

    text(0.5, p:1,
         paste(labels, "  ", sep = ""),
         adj = 1,
         cex = cex.labels)

    legend(x = "bottomright",
           legend = mod.names,
           lwd = n.models:1,
           col = 1:n.models,
           lty = 1:n.models,
           bty = "n",
           horiz = TRUE)
  }

  invisible(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.lmfm.R
\name{summary.lmfm}
\alias{summary.lmfm}
\title{Comparison Summaries for Linear Regression Models}
\usage{
\method{summary}{lmfm}(object, correlation = FALSE, ...)
}
\arguments{
\item{object}{an \code{lmfm} object.}

\item{correlation}{a logical value.  If TRUE, the correlation matrices for
the coefficients are included in the summaries.}

\item{\dots}{additional arguments required by the generic
\code{\link{summary}} function.}
}
\value{
a list with class summary.lmfm whose elements are summaries of each
model in \code{object}.
}
\description{
Compute a summary of each model in an \code{lmfm} object.
}
\examples{

data(stackloss)
m1 <- lm(stack.loss ~ ., data = stackloss)
m2 <- lm(stack.loss ~ . - Acid.Conc., data = stackloss)

fm <- fit.models(m1, m2)
print(fm.sum <- summary(fm))
}
\keyword{methods}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.covfm.R
\name{summary.covfm}
\alias{summary.covfm}
\title{Summary Method}
\usage{
\method{summary}{covfm}(object, corr = FALSE, ...)
}
\arguments{
\item{object}{a \dQuote{covfm} object.}

\item{corr}{a logical value passed as an attribute to the \code{print}
method. When \code{TRUE}, correlations are compared in the
textual output.}

\item{\dots}{additional arguments for the summary method.}
}
\description{
Generic summary method for \dQuote{covfm} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/designMD.R
\name{designMD}
\alias{designMD}
\alias{designMD.default}
\title{Design Matrix Mahalanobis Distance}
\usage{
designMD(object, ...)
}
\arguments{
\item{object}{a fitted model object with a \code{\link{model.matrix}} method.}

\item{\dots}{additional arguments are ignored.}
}
\value{
a numeric vector containing the squared Mahalanobis distances.
}
\description{
Returns the squared Mahalanobis distance of all rows in the design (model)
matrix \eqn{X} and the sample mean vector \eqn{\mu} of the columns
of \eqn{X} with respect to the sample covariance matrix \eqn{\Sigma}.
This is (for vector \eqn{x'} a row of \eqn{X}) defined as
\deqn{d^{2} = (x - \mu)' \Sigma^{-1} (x - \mu)}
where
\deqn{\mu = colMeans(X)}
and
\deqn{\Sigma = cov(X).}
}
\examples{
stack.lm <- lm(stack.loss ~ ., data = stackloss)

# Mahalanobis distance (not squared)
sqrt(designMD(stack.lm))
}
\keyword{methods}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sideBySideKernelDensityPlot.R
\name{sideBySideKernelDensityPlot}
\alias{sideBySideKernelDensityPlot}
\title{Comparison Kernel Density Estimate Plot}
\usage{
sideBySideKernelDensityPlot(x, fun, ...)
}
\arguments{
\item{x}{a \code{fit.models} object.}

\item{fun}{a function to extract the desired quantity from \code{x}.}

\item{\dots}{additional arguments are passed to
\code{\link[lattice]{xyplot}}.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Produces side-by-side kernel density estimate plots.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.glmfm.R
\name{plot.glmfm}
\alias{plot.glmfm}
\title{Comparison Diagnostic Plots for Generalized Linear Models}
\usage{
\method{plot}{glmfm}(x, which.plots = 1:6, ...)
}
\arguments{
\item{x}{a \code{glmfm} object.}

\item{which.plots}{either \code{"ask"} (character string) or an integer
vector specifying which plots to draw. In the later case, the plot numbers
are given above.}

\item{\dots}{other parameters to be passed through to plotting functions.}
}
\value{
\code{x} is invisibly returned.
}
\description{
Produces a set of comparison diagnostic plots.  The plot options are
\enumerate{
  \item Deviance Residuals vs. Predicted Values,
  \item Response vs. Fitted Values,
  \item Normal QQ Plot of Pearson Residuals,
  \item Normal QQ Plot of Deviance Residuals,
  \item Pearson Residuals vs. Mahalanobis Distance,
  \item Sqrt Deviance Residuals vs. Predicted Values.
}
}
\section{Side Effects}{
 The selected plots are drawn on a graphics device.
}

\examples{
# From ?glm:
# A Gamma example, from McCullagh & Nelder (1989, pp. 300-2)

clotting <- data.frame(
    u = c(5,10,15,20,30,40,60,80,100),
    lot1 = c(118,58,42,35,27,25,21,19,18),
    lot2 = c(69,35,26,21,18,16,13,12,12))

lot1 <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
lot2 <- glm(lot2 ~ log(u), data = clotting, family = Gamma)

fm <- fit.models(lot1, lot2)
plot(fm)

}
\seealso{
\code{\link{sideBySideQQPlot}} for 4 and 5 and
\code{\link{sideBySideScatterPlot}} for the others.
}
\keyword{hplot}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddPlot.covfm.R
\name{ddPlot.covfm}
\alias{ddPlot.covfm}
\title{Distance - Distance Plot}
\usage{
ddPlot.covfm(x, level = 0.95, strip = "", id.n = 3, ...)
}
\arguments{
\item{x}{a \code{"covfm"} object.}

\item{level}{a single numeric value between 0 and 1 giving the
chi-squared percent point used to compute the outlyingness threshold.}

\item{strip}{a character string printed in the \dQuote{strip} at the top
of the plot.}

\item{id.n}{a single nonnegative integer specifying the number of
extreme points to label in the plot.}

\item{...}{additional arguments are passed to \code{xyplot}.}
}
\value{
if the models can be compared then the plotted \code{trellis} object is
        invisibly returned.  Otherwise \code{x} is returned invisibly.
}
\description{
For a \code{covfm} object containing 2 models, this function plots the
             Mahalanobis distance from the first model on the y-axis and the
             Mahalanobis distance from the second model on the x-axis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covfm.R
\name{center}
\alias{center}
\title{Calculate Location Estimate for a Fitted Model Object}
\usage{
center(object, ...)
}
\arguments{
\item{object}{a fitted model object, typically.  Sometimes
also a \code{summary()} object of such a fitted
model.}

\item{\dots}{additional arguments for method functions.}
}
\description{
Returns the location estimated from a
             location/scatter-type fitted model object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaidQQPlot.R
\name{overlaidQQPlot}
\alias{overlaidQQPlot}
\title{Overlaid Normal QQ Plot}
\usage{
overlaidQQPlot(x, fun, ...)
}
\arguments{
\item{x}{a \code{fit.models} object.}

\item{fun}{a function to extract the desired quantity from \code{x}.}

\item{\dots}{additional arguments are passed to
\code{\link[lattice]{qqmath}}.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Produces an overlaid normal QQ plot.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sideBySideScatterPlot.R
\name{sideBySideScatterPlot}
\alias{sideBySideScatterPlot}
\title{Comparison Scatter Plot}
\usage{
sideBySideScatterPlot(object, x.fun, y.fun, ...)
}
\arguments{
\item{object}{a \code{fit.models} object.}

\item{x.fun}{a function to extract the x-axis quantity.}

\item{y.fun}{a function to extract the y-axis quantity.}

\item{\dots}{additional arguments.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Produces a side-by-side scatter plot.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.glmfm.R
\name{summary.glmfm}
\alias{summary.glmfm}
\title{Comparison Summaries for Generalized Linear Models}
\usage{
\method{summary}{glmfm}(object, correlation = FALSE, ...)
}
\arguments{
\item{object}{a glmfm object.}

\item{correlation}{a logical value. If \code{TRUE}, correlation matrices of
the coefficient estimates are included in each summary.}

\item{\dots}{additional arguments required by the generic
\code{\link{summary}} function.}
}
\value{
a list with class summary.glmfm whose elements are summaries of each
model in \code{object}.
}
\description{
Compute a summary of each model in a \code{glmfm} object.
}
\examples{

# From ?glm:
# A Gamma example, from McCullagh & Nelder (1989, pp. 300-2)

clotting <- data.frame(
    u = c(5,10,15,20,30,40,60,80,100),
    lot1 = c(118,58,42,35,27,25,21,19,18),
    lot2 = c(69,35,26,21,18,16,13,12,12))

lot1 <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
lot2 <- glm(lot2 ~ log(u), data = clotting, family = Gamma)

fm <- fit.models(lot1, lot2)
summary(fm)
}
\keyword{methods}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distancePlot.covfm.R
\name{distancePlot.covfm}
\alias{distancePlot.covfm}
\title{Side-by-Side Mahalanobis Distance Plot}
\usage{
distancePlot.covfm(x, level = 0.95, id.n = 3, ...)
}
\arguments{
\item{x}{a \code{"covfm"} object.}

\item{level}{a single numeric value between 0 and 1 giving the chi-squared percent
point used to compute the outlyingness threshold.}

\item{id.n}{a single nonnegative integer specifying the number of extreme points to
label in the plot.}

\item{\dots}{additional arguments are passed to \code{xyplot}.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Produces side-by-side plots of Mahalanobis distance computed using the
             location and covariance matrix estimates contained in each element of a
             \code{covfm} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screePlot.covfm.R
\name{screePlot.covfm}
\alias{screePlot.covfm}
\title{Comparison Screeplot}
\usage{
screePlot.covfm(x, npcs, strip = "", ...)
}
\arguments{
\item{x}{a \code{"covfm"} object.}

\item{npcs}{a postive integer value specifying the number of components to be plotted.}

\item{strip}{a character string printed in the \dQuote{strip} at the top of the plot.}

\item{...}{additional arguments are passed to \code{xyplot}.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Overlaid screeplot for the models in a \dQuote{covfm} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sideBySideIndexPlot.R
\name{sideBySideIndexPlot}
\alias{sideBySideIndexPlot}
\title{Comparison Index (Time) Plot}
\usage{
sideBySideIndexPlot(x, fun, level = 0.95, id.n = 3, ...)
}
\arguments{
\item{x}{a \code{fit.models} object.}

\item{fun}{a function to extract the desired quantity from \code{x}.}

\item{level}{a numeric value between 0 and 1 specifying the confidence level
used to draw the threshold in the plot.}

\item{id.n}{a non-negative integer value specifying the number of extreme
points to identify.}

\item{\dots}{any additional arguments are passed to
\code{\link[lattice]{xyplot}}.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Produces side-by-side index (time) plots.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaidKernelDensityPlot.R
\name{overlaidKernelDensityPlot}
\alias{overlaidKernelDensityPlot}
\title{Overlaid Kernel Density Estimate Plot}
\usage{
overlaidKernelDensityPlot(x, fun, ...)
}
\arguments{
\item{x}{a \code{fit.models} object.}

\item{fun}{a function to extract the desired quantity from \code{x}.}

\item{\dots}{additional arguments are passed to
\code{densityplot}.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Produces an overlaid kernel density plot.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.models.R
\name{fit.models}
\alias{fit.models}
\title{Fit dot Models}
\usage{
fit.models(model.list, ...)
}
\arguments{
\item{model.list}{a list or a character vector containing names of modeling
functions.  Only required when \code{fit.models} is being used to fit models
(rather than combine already fitted models into a \code{fit.models} object).}

\item{\dots}{see details.}
}
\value{
The returned object is a list containing the fitted models.  The
class of the retuned object depends on the classes of the model objects it
contains.
}
\description{
Fit a statistical model using different estimators (e.g., robust and
least-squares) or combine fitted models into a single object.  Generic
methods then produce side-by-side comparisons of the parameter estimates and
diagnostic plots.
}
\details{
There are two distinct ways the \code{fit.models} function can be used.

The first is to fit the same model using different estimators.  In this
case, \code{model.list} should be a character vector or a list where each
element is the name of a modeling function and the remaining arguments (in
\dots) are the common arguments to the functions in \code{model.list}.
For example, the following command fits robust and least squares linear
models to Brownlee's Stack Loss Plant Data. \preformatted{
fit.models(c("rlm", "lm"), stack.loss ~ ., data = stackloss)} The resulting
\code{fit.models} object is a list with the output of \preformatted{
rlm(stack.loss ~ ., data = stackloss)} in the first element and
\preformatted{ lm(stack.loss ~ ., data = stackloss)} in the second.  The
class attribute of the returned list is set (in this case) to \code{"lmfm"}
which is the \code{fit.models} class (fmclass) for comparing linear-model-like
fits.

The second use of fit.models is to combine fitted model objects.  In
this case, \code{fit.models} combines its arguments into a fit.models object
(a list where element \eqn{i} is occupied by argument \eqn{i} and sets the
class attribute to the appropriate \code{fit.models} class.
}
\examples{
  # First, use fit.models to fit robust and least squares linear
  # regression models to Brownlee's Stack Loss Plant Data.

  # Step 1: rlm (robust linear model) is in the MASS package.
  library(MASS)

  # Step 2: tell fit.models rlm can be compared to lm
  fmclass.add.class("lmfm", "rlm")

  fm1 <- fit.models(c("rlm", "lm"), stack.loss ~ ., data = stackloss)

  summary(fm1) #rlm does not provide p-values or Multiple R-squared


  # Second, use fit.models to combine fitted models into a
  # fit.models object.

  lm.complete <- lm(stack.loss ~ ., data = stackloss)
  lm.clean <- lm(stack.loss ~ ., data = stackloss, subset = 5:20)

  fm2 <- fit.models(lm.clean, lm.complete)

  summary(fm2)
  plot(fm2)


  # Name the models in the fit.models object.
  
  fm3 <- fit.models(c(Robust = "rlm", "Least Squares" = "lm"),
                    stack.loss ~ ., data = stackloss)

  fm4 <- fit.models(Clean = lm.clean, Complete = lm.complete)
}
\seealso{
\code{\link{fmclass.add.class}} for adding a class to an existing
fit.models class and \code{\link{fmclass.register}} to create a new
fit.models class.
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaidSimpleRegressionPlot.R
\name{overlaidSimpleRegressionPlot}
\alias{overlaidSimpleRegressionPlot}
\title{Scatter Plot with Overlaid Fits}
\usage{
overlaidSimpleRegressionPlot(x, lwd.reg, col.reg, ...)
}
\arguments{
\item{x}{a \code{fit.models} object.}

\item{lwd.reg}{a vector with length equal to the number of models in
\code{x} specifying the line widths used in the plot.}

\item{col.reg}{a vector with length equal to the number of models in
\code{x} specifying the line colors used in the plot.}

\item{\dots}{additional arguments are passed to
\code{\link[lattice]{xyplot}}.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Produces a scatter plot of the data with overlaid fits.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sideBySideQQPlot.R
\name{sideBySideQQPlot}
\alias{sideBySideQQPlot}
\title{Compaison QQ Plot}
\usage{
sideBySideQQPlot(
  x,
  fun,
  envelope = TRUE,
  half.normal = FALSE,
  n.samples = 250,
  level = 0.95,
  id.n = 3,
  qqline = TRUE,
  ...
)
}
\arguments{
\item{x}{a \code{fit.models} object.}

\item{fun}{a function to extract the desired quantity from \code{x}.}

\item{envelope}{a logical value.  If \code{TRUE} a \code{level} confidence
envelope is simulated for each QQ plot.}

\item{half.normal}{a logical value.  If \code{TRUE} the plot is drawn using
the absolute values.}

\item{n.samples}{a positive integer value giving the number of samples to
compute in the simulation of the confidence envelope.}

\item{level}{a numeric value between 0 and 1 specifying the confidence level
for the envelope.}

\item{id.n}{a non-negative integer value specifying the number of extreme
points to identify.}

\item{qqline}{a logical value.  If \code{TRUE}, a QQ line is included in the
plot.}

\item{\dots}{additional arguments are passed to
\code{\link[lattice]{xyplot}}.}
}
\value{
the \code{trellis} object is invisibly returned.
}
\description{
Produces side-by-side QQ plots.  An optional simulated confidence envelope
can be included in each plot.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.lmfm.R
\name{plot.lmfm}
\alias{plot.lmfm}
\title{Comparison Diagnostic Plots for Linear Regression Models}
\usage{
\method{plot}{lmfm}(x, which.plots = 1:10, ...)
}
\arguments{
\item{x}{an \code{lmfm} object.}

\item{which.plots}{either \code{"ask"} (character string) or an integer
vector specifying which plots to draw. In the later case, the plot numbers
are given above.}

\item{\dots}{additional parameters are ignored.}
}
\value{
\code{x} is invisibly returned.
}
\description{
Produces a set of comparison diagnostic plots.  The plot options are
\enumerate{
  \item Normal QQ Plot of Residuals,
  \item Kernel Density Estimate of Residuals,
  \item Residuals vs. Mahalanobis Distance,
  \item Residuals vs. Fitted Values,
  \item Sqrt Residuals vs. Fitted Values,
  \item Response vs. Fitted Values,
  \item Residuals vs. Index (Time),
  \item Overlaid Normal QQ Plot of Residuals,
  \item Overlaid Kernel Density Estimate of Residuals,
  \item Scatter Plot with Overlaid Fits (for simple linear regression models).
  }
}
\section{Side Effects}{
 The selected plots are drawn on a graphics device.
}

\examples{
data(stackloss)
stack.lm <- lm(stack.loss ~ ., data = stackloss)
stack.clean <- lm(stack.loss ~ ., data = stackloss, subset = 5:20)
fm <- fit.models(stack.clean, stack.lm)
plot(fm)
}
\seealso{
See \code{\link{sideBySideQQPlot}} for 2,
\code{\link{sideBySideKernelDensityPlot}} for 3,
\code{\link{sideBySideIndexPlot}} for 8,
\code{\link{overlaidQQPlot}} for 9,
\code{\link{overlaidKernelDensityPlot}} for 10,
\code{\link{overlaidSimpleRegressionPlot}} for 11, and
\code{\link{sideBySideScatterPlot}} for the others.
}
\keyword{hplot}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covfm.R
\name{vcor}
\alias{vcor}
\title{Calculate Correlation Matrix for a Fitted Model Object}
\usage{
vcor(object, ...)
}
\arguments{
\item{object}{a fitted model object, typically.  Sometimes
also a \code{summary()} object of such a fitted
model.}

\item{\dots}{additional arguments for method functions.}
}
\description{
Retrieve a correlation matrix estimate from a
             fitted model object. The default method uses
             \code{\link[stats]{cov2cor}} to scale the
             covariance matrix returned by
             \code{\link[stats]{vcov}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fmclass.register.R
\name{fmclass.register}
\alias{fmclass.register}
\alias{fmclass.add.class}
\title{Register Comparable Functions}
\usage{
fmclass.register(fmclass, classes, validation.function = NULL)

fmclass.add.class(fmclass, class, warn = TRUE)
}
\arguments{
\item{fmclass}{a character string naming the fit.models class to be added.}

\item{classes}{a character vector naming one or more classes that can be
compared by the methods defined for the fit.models class in \code{fmclass}.}

\item{validation.function}{a function returning \code{TRUE} when the models
are comparable.}

\item{class}{a character string specifying a class compatible with the
methods of \code{fmclass}.}

\item{warn}{a logical value. If TRUE, a warning is printed if \code{class}
is already registered.}
}
\value{
a null value is invisibly returned.
}
\description{
The \code{fit.models} package maintains a list of comparable models.
These functions provide an api to modify this list.
}
\details{
See the package vignette.
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.covfm.R
\name{plot.covfm}
\alias{plot.covfm}
\title{Plot Method}
\usage{
\method{plot}{covfm}(x, which.plots = 1:4, ...)
}
\arguments{
\item{x}{a \code{covfm} object.}

\item{which.plots}{either \code{"ask"} (character string) or an integer vector specifying
which plots to draw.  The plot options are (1) Mahalanobis Distance,
(2) Ellipses Matrix, (3) Screeplot (Eigenvalues of Covariance Estimate),
and (4) Distance - Distance Plot.}

\item{...}{additional arguments are passed to the plot subfunctions.}
}
\value{
\code{x} is returned invisibly.
}
\description{
Generic plot method for \dQuote{covfm} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ellipsesPlot.covfm.R
\name{ellipsesPlot.covfm}
\alias{ellipsesPlot.covfm}
\title{Ellipses Plot - Visual Correlation Matrix Comparison}
\usage{
ellipsesPlot.covfm(x, ...)
}
\arguments{
\item{x}{a \code{"covfm"} object.}

\item{\dots}{additional arguments are ignored.}
}
\value{
x is invisibly returned.
}
\description{
When there are 3 or more variables in the data, this function produces
             a matrix with ellipses drawn in the upper triangle.  The ellipse in cell
             \eqn{i,j} of the plot is drawn to be a contour of a standard bivariate
             normal with correlation \eqn{\rho_{ij}}.  One ellipse is drawn in each
             cell for each model in the \code{covfm} object.  When there are 2
             variables in the data, this function produces a scatter plot of the data
             with an overlaid 95\% confidence ellipse for each model in the
             \code{covfm} object.
}
